#include <J3ML/LinearAlgebra/AxisAngle.h>
#include <J3ML/LinearAlgebra/Quaternion.h>
namespace J3ML::LinearAlgebra {

    AxisAngle::AxisAngle() : axis(Vector3::Zero) {}

    AxisAngle::AxisAngle(const Vector3 &axis, float angle) : axis(axis), angle(angle) {}

    Quaternion AxisAngle::ToQuaternion() const {
        return {
                axis.x * std::sin(angle/2),
                axis.y * std::sin(angle/2),
                axis.z * std::sin(angle/2),
                std::cos(angle/2)
        };
    }

    AxisAngle::AxisAngle(const Quaternion &q) {
        auto theta = std::acos(q.w) * 2.f;
        auto ax = q.x / std::sin(std::acos(theta));
        auto ay = q.y / std::sin(std::acos(theta));
        auto az = q.z / std::sin(std::acos(theta));
    }

    AxisAngle::AxisAngle(const EulerAngle &e) {

        // Assuming the angles are in radians

        float heading = e.pitch;
        float attitude = e.yaw;
        float bank = e.roll;

        float c1 = std::cos(heading / 2.f);
        float s1 = std::sin(heading / 2.f);
        float c2 = std::cos(attitude / 2.f);
        float s2 = std::sin(attitude / 2.f);
        float c3 = std::cos(bank / 2.f);
        float s3 = std::sin(bank / 2.f);

        float w = c1*c2*c3 - s1*s2*s3;
        float x = c1*c2*c3 + s1*s2*s3;
        float y = s1*c2*c3 + c1*s2*s3;
        float z = c1*s2*c3 - s1*c2*s3;

        angle = 2.f * std::acos(w);

        double norm = x*x + y*y + z*z;
        if (norm < 0.001) { // when all euler angles are zero angle=0, so
            // we can set axis to anything to avoid divide by zero
            x = 1;
            y = z = 0;
        } else {
            norm = std::sqrt(norm);
            x /= norm;
            y /= norm;
            z /= norm;
        }

        axis = {x, y, z};
    }
}