#include <J3ML/LinearAlgebra/EulerAngle.h>
#include <cmath>
#include <algorithm>


namespace J3ML::LinearAlgebra {
    EulerAngle::EulerAngle(float pitch, float yaw, float roll): pitch(pitch), yaw(yaw), roll(roll)
    {}

    float EulerAngle::GetPitch(float pitch_limit) const
    { return std::clamp( std::remainderf(pitch,360.f), -pitch_limit, pitch_limit); }

    float EulerAngle::GetYaw(float yaw_limit) const
    { return std::clamp(std::remainderf(yaw, 360.f), -yaw_limit, yaw_limit);   }

    float EulerAngle::GetRoll(float pitch_limit) const
    { return std::clamp( std::remainderf(pitch,360.f), -pitch_limit, pitch_limit); }

    bool EulerAngle::operator==(const EulerAngle& a) const
    {
        return (pitch == a.pitch) && (yaw == a.yaw) && (roll == a.roll);
    }

    void EulerAngle::clamp()
    {
        if (this->pitch > 89.0f)
            this->pitch = 89.0f;
        if (this->pitch <= -89.0f)
            this->pitch = -89.0f;
        //TODO: Make this entirely seamless by getting the amount they rotated passed -180 and +180 by.
        if (this->yaw <= -180.0f)
            this->yaw = 180.0f;
        if (this->yaw >= 180.01f)
            this->yaw = -179.9f;
        if (this->roll >= 360.0f)
            this->roll = 0.0;
        if (this->roll <= -360.0f)
            this->roll = 0.0;
    }

    EulerAngle EulerAngle::movementAngle() const
    {
        EulerAngle a;
        a.pitch = (cos(Math::Radians(yaw)) * cos(Math::Radians(pitch)));
        a.yaw = -sin(Math::Radians(pitch));
        a.roll = (sin(Math::Radians(yaw)) * cos(Math::Radians(pitch)));
        return a;
    }

    EulerAngle::EulerAngle() : pitch(0), yaw(0), roll(0) {}

    EulerAngle::EulerAngle(const AxisAngle &rhs) {

        float x = rhs.axis.x;
        float y = rhs.axis.y;
        float z = rhs.axis.z;
        float angle = rhs.angle;

        double s = std::sin(rhs.angle);

        double c = std::cos(rhs.angle);

        double t = 1-c;

        // if axis is not already normalized then uncomment this

        // double magnitude = std::sqrt(x*x + y*y + z*z);
        // if (magnitude == 0) throw error;
        // x /= magnitude;
        // y /= magnitude;
        // z /= magnitude;

        if ((x*y*t + z*s) > 0.998) { // North pole singularity detected
            pitch = 2 * std::atan2(x * std::sin(angle/2.f), std::cos(angle/2.f));
            yaw = M_PI / 2.f;
            roll = 0;
            return;
        }

        if ((x*y*t + z*s) < -0.998) { // South pole singularity detected
            pitch = -2 * std::atan2(x * std::sin(angle/2.f), std::cos(angle/2.f));
            yaw = -M_PI / 2.f;
            roll = 0;
            return;
        }

        pitch = std::atan2(y * s-x * z * t, 1 - (y*y + z*z) * t);
        yaw = std::asin(x * y * t + z * s);
        roll = std::atan2(x * s - y * z * t, 1 - (x*x + z*z) * t);
    }

    EulerAngle::EulerAngle(const Quaternion &rhs) {
        double test = rhs.x * rhs.y + rhs.z * rhs.w;
        if (test > 0.499) { // Singularity at north pole
            pitch = 2 * std::atan2(rhs.x, rhs.w);
            yaw = M_PI / 2.f;
            roll = 0;
            return;
        }

        if (test < -0.499) { // Singularity at south pole
            pitch = -2 * std::atan2(rhs.x, rhs.y);
            yaw = - M_PI / 2.f;
            roll = 0;
            return;
        }
        float sqx = rhs.x * rhs.x;
        float sqy = rhs.y * rhs.y;
        float sqz = rhs.z * rhs.z;
    }
}