#include <gtest/gtest.h>
#include <J3ML/LinearAlgebra/Vector2.h>

using J3ML::LinearAlgebra::Vector2;

TEST(Vector2Test, V2_Constructor_Default)
{
    EXPECT_EQ(Vector2(), Vector2::Zero);
}

TEST(Vector2Test, V2_Constructor_XY)
{
    Vector2 vec {1, 0};
    EXPECT_EQ(vec, Vector2::Right);
}

TEST(Vector2Test, V2_Addition_Op)
{
    Vector2 A {1,1};
    Vector2 B {2,2};

    Vector2 C {3, 3};

    EXPECT_EQ(A+B, C);
}

TEST(Vector2Test, V2_Addition_Method)
{
    Vector2 A {2,2};
    Vector2 B {2,2};

    Vector2 C {4, 4};

    EXPECT_EQ(A.Add(B), C);
}

TEST(Vector2Test, V2_Addition_Static)
{
    Vector2 A {3, 3};
    Vector2 B {2, 2};

    Vector2 C {5, 5};

    EXPECT_EQ(Vector2::Add(A, B), C);
}

TEST(Vector2Test, V2_Subtract_Op)
{
    Vector2 A {1,1};
    Vector2 B {2,2};

    Vector2 C {-1, -1};

    EXPECT_EQ(A-B, C);
}

TEST(Vector2Test, V2_Subtract_Method)
{
    Vector2 A {1,1};
    Vector2 B {2,2};

    Vector2 C {-1, -1};

    EXPECT_EQ(A.Sub(B), C);
}

TEST(Vector2Test, V2_Subtract_Static)
{
    Vector2 A {1,1};
    Vector2 B {2,2};

    Vector2 C {-1, -1};

    EXPECT_EQ(Vector2::Sub(A, B), C);
}

TEST(Vector2Test, V2_Scalar_Multiplication)
{
    Vector2 A {5, 1};
    float B = 0.5f;
    Vector2 C = {2.5f, .5f};
    EXPECT_EQ(A*B, C);
}

TEST(Vector2Test, V2_Size)
{
    EXPECT_EQ(sizeof(Vector2), 8);
}

TEST(Vector2Test, V2_NaN)
{

    EXPECT_NE(Vector2::Zero, Vector2::NaN);
    EXPECT_NE(Vector2::Up, Vector2::NaN);
    EXPECT_NE(Vector2::Left, Vector2::NaN);
    EXPECT_NE(Vector2::Down, Vector2::NaN);
    EXPECT_NE(Vector2::Right, Vector2::NaN);
    EXPECT_NE(Vector2::NaN, Vector2::NaN);
}

TEST(Vector2Test, V2_MarginOfError)
{
    Vector2 A {2,2};
    Vector2 B {1.85, 1.85};

    EXPECT_TRUE(A.IsWithinMarginOfError(B, 0.5f));
}

TEST(Vector2Test, V2_Min)
{
    Vector2 A {2,2};
    Vector2 B {1.85, 1.85};

    EXPECT_EQ( Vector2::Min(A, B), B);
}

TEST(Vector2Test, V2_Max)
{
    Vector2 A {2,2};
    Vector2 B {1.85, 1.85};

    EXPECT_EQ( Vector2::Max(A, B), A);
}

TEST(Vector2Test, V2_Clamp)
{
    Vector2 Input{0, 20};

    Vector2 Minimum { 2, 2};
    Vector2 Maximum {16, 16};

    Vector2 ExpectedResult {2, 16};

    EXPECT_EQ(Input.Clamp(Minimum, Maximum), ExpectedResult);
}

TEST(Vector2Test, V2_DotProduct)
{
    // TODO: Equality
    Vector2 A {2, 2};
    Vector2 B {1, 1};
    EXPECT_FLOAT_EQ(A.Dot(B), 4.f);
}

TEST(Vector2Test, V2_Project)
{
    Vector2 Base {4, 4};
    Vector2 Projected {1, 0};

    Vector2 ExpectedResult {4, 0};

    EXPECT_EQ(Base.Project(Projected), ExpectedResult);
}

TEST(Vector2Test, V2_Normalize)
{
    Vector2 A{2, 0};
    Vector2 B{1, 0};
    EXPECT_EQ(A.Normalize(), B);
}

TEST(Vector2Test, V2_Lerp)
{
    Vector2 A {2,2};
    Vector2 B {10, 10};
    Vector2 C {6, 6};

    EXPECT_EQ(A.Lerp(B, 0.f), A);
    EXPECT_EQ(A.Lerp(B, 1.f), B);
    EXPECT_EQ(A.Lerp(B, 0.5f), C);
}

TEST(Vector2Test, V2_AngleBetween)
{
    Vector2 A {0.5f, 0.5};
    Vector2 B {0.5f, 0.1f};

    A = A.Normalize();
    B = B.Normalize();

    // TODO: AngleBetween returns not a number
    EXPECT_FLOAT_EQ(A.AngleBetween(B), 0.58800244);
}